#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>

from mtgsdk.querybuilder import QueryBuilder


class Card(object):
    RESOURCE = 'cards'

    def __init__(self, response_dict={}):
        self.name = response_dict.get('name')
        self.layout = response_dict.get('layout')
        self.mana_cost = response_dict.get('manaCost')
        self.cmc = response_dict.get('cmc')
        self.colors = response_dict.get('colors')
        self.color_identity = response_dict.get('colorIdentity')
        self.names = response_dict.get('names')
        self.type = response_dict.get('type')
        self.supertypes = response_dict.get('supertypes')
        self.subtypes = response_dict.get('subtypes')
        self.types = response_dict.get('types')
        self.rarity = response_dict.get('rarity')
        self.text = response_dict.get('text')
        self.flavor = response_dict.get('flavor')
        self.artist = response_dict.get('artist')
        self.number = response_dict.get('number')
        self.power = response_dict.get('power')
        self.toughness = response_dict.get('toughness')
        self.loyalty = response_dict.get('loyalty')
        self.multiverse_id = response_dict.get('multiverseid')
        self.variations = response_dict.get('variations')
        self.watermark = response_dict.get('watermark')
        self.border = response_dict.get('border')
        self.timeshifted = response_dict.get('timeshifted')
        self.hand = response_dict.get('hand')
        self.life = response_dict.get('life')
        self.release_date = response_dict.get('releaseDate')
        self.starter = response_dict.get('starter')
        self.printings = response_dict.get('printings')
        self.original_text = response_dict.get('originalText')
        self.original_type = response_dict.get('originalType')
        self.source = response_dict.get('source')
        self.image_url = response_dict.get('imageUrl')
        self.set = response_dict.get('set')
        self.set_name = response_dict.get('setName')
        self.id = response_dict.get('id')
        self.legalities = response_dict.get('legalities')
        self.rulings = response_dict.get('rulings')
        self.foreign_names = response_dict.get('foreignNames')

    @staticmethod
    def find(id):
        return QueryBuilder(Card).find(id)

    @staticmethod
    def where(**kwargs):
        return QueryBuilder(Card).where(**kwargs)

    @staticmethod
    def all():
        return QueryBuilder(Card).all()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>

import vcr
import unittest
from mtgsdk import Card

# Python 3.6 Workaround until https://github.com/kevin1024/vcrpy/issues/293 is fixed.
vcr_connection_request = vcr.stubs.VCRConnection.request 
vcr.stubs.VCRConnection.request = lambda *args, **kwargs: vcr_connection_request(*args)

class TestCard(unittest.TestCase):
    def test_find_returns_card(self):
        with vcr.use_cassette('fixtures/choice_of_damnations.yaml'):
            card = Card.find(88803)

            self.assertEqual('Choice of Damnations', card.name)
            self.assertEqual('{5}{B}', card.mana_cost)
            self.assertEqual(6, card.cmc)
            self.assertEqual('Sorcery — Arcane', card.type)
            self.assertTrue('Black' in card.colors)
            self.assertEqual(['B'], card.color_identity)
            self.assertTrue('Sorcery' in card.types)
            self.assertTrue('Arcane' in card.subtypes)
            self.assertEqual('Rare', card.rarity)
            self.assertEqual('SOK', card.set)
            self.assertEqual('Saviors of Kamigawa', card.set_name)
            self.assertEqual("Target opponent chooses a number. You may have that player lose that much life. If you don't, that player sacrifices all but that many permanents.", card.text)
            self.assertEqual("\"Life is a series of choices between bad and worse.\"\n—Toshiro Umezawa", card.flavor)
            self.assertEqual('Tim Hildebrandt', card.artist)
            self.assertEqual('62', card.number)
            self.assertEqual(88803, card.multiverse_id)
            self.assertEqual('http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88803&type=card', card.image_url)
            self.assertTrue(len(card.rulings) > 0)
            self.assertTrue({"name":"Scelta della Dannazione","language":"Italian","multiverseid":105393, "imageUrl":"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=105393&type=card"} in card.foreign_names)
            self.assertTrue('SOK' in card.printings)
            self.assertEqual("Target opponent chooses a number. You may have that player lose that much life. If you don't, that player sacrifices all but that many permanents.", card.original_text)
            self.assertEqual('Sorcery — Arcane', card.original_type)            
            self.assertTrue({"format":"Commander","legality":"Legal"} in card.legalities)
            self.assertEqual('1c4aab072d52d283e902f2302afa255b39e0794b', card.id)

    def test_all_with_params_return_cards(self):
        with vcr.use_cassette('fixtures/legendary_elf_warriors.yaml'):
            cards = Card.where(supertypes='legendary') \
                        .where(subtypes='elf,warrior') \
                        .all()
            
            self.assertTrue(len(cards) >= 13)
            
    def test_all_with_page_returns_cards(self):
        with vcr.use_cassette('fixtures/all_first_page.yaml'):
            cards = Card.where(page=1).all()
            
            self.assertEqual(100, len(cards))
            
    def test_all_with_page_and_page_size_returns_card(self):
        with vcr.use_cassette('fixtures/all_first_page_one_card.yaml'):
            cards = Card.where(page=1).where(pageSize=1).all()
            
            self.assertEqual(1, len(cards))


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>

import unittest
from mtgsdk import __version__, __pypi_packagename__, __github_username__, __github_reponame__, __endpoint__

class TestConfig(unittest.TestCase):
    def test_has_proper_version(self):
        self.assertEqual('1.3.1', __version__)
        
    def test_has_proper_packagename(self):
        self.assertEqual('mtgsdk', __pypi_packagename__)
        
    def test_has_proper_github_username(self):
        self.assertEqual('MagicTheGathering', __github_username__)
        
    def test_has_proper_github_reponame(self):
        self.assertEqual('mtg-sdk-python', __github_reponame__)
        
    def test_has_proper_endpoint(self):
        self.assertEqual('https://api.magicthegathering.io/v1', __endpoint__)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>

from mtgsdk.querybuilder import QueryBuilder
from mtgsdk.config import __endpoint__
from mtgsdk.card import Card


class Set(object):
    RESOURCE = 'sets'

    def __init__(self, response_dict={}):
        self.code = response_dict.get('code')
        self.name = response_dict.get('name')
        self.type = response_dict.get('type')
        self.border = response_dict.get('border')
        self.mkm_id = response_dict.get('mkm_id')
        self.mkm_name = response_dict.get('mkm_name')
        self.release_date = response_dict.get('releaseDate')
        self.gatherer_code = response_dict.get('gathererCode')
        self.magic_cards_info_code = response_dict.get('magicCardsInfoCode')
        self.booster = response_dict.get('booster')
        self.old_code = response_dict.get('oldCode')
        self.block = response_dict.get('block')
        self.online_only = response_dict.get('onlineOnly')

    @staticmethod
    def find(id):
        return QueryBuilder(Set).find(id)

    @staticmethod
    def where(**kwargs):
        return QueryBuilder(Set).where(**kwargs)

    @staticmethod
    def all():
        return QueryBuilder(Set).all()
        
    @staticmethod
    def generate_booster(code):
        url = "{}/{}/{}/booster".format(__endpoint__, Set.RESOURCE, code)
        return QueryBuilder(Set).find_many(url, Card, Card.RESOURCE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>

import vcr
import unittest
from mtgsdk import Set

class TestSet(unittest.TestCase):
    def test_find_returns_set(self):
        with vcr.use_cassette('fixtures/ktk.yaml'):
            set = Set.find('ktk')
            
            self.assertEqual('KTK', set.code)
            self.assertEqual('Khans of Tarkir', set.name)
            self.assertEqual('expansion', set.type)
            self.assertEqual('black', set.border)
            self.assertTrue('common' in set.booster)
            self.assertEqual('2014-09-26', set.release_date)
            self.assertEqual('ktk', set.magic_cards_info_code)
            
    def test_generate_booster_returns_cards(self):
        with vcr.use_cassette('fixtures/booster.yaml'):
            cards = Set.generate_booster('ktk')
            
            self.assertEqual(15, len(cards))
            self.assertEqual('KTK', cards[0].set)
            
    def test_where_filters_on_name(self):
        with vcr.use_cassette('fixtures/filtered_sets.yaml'):
            sets = Set.where(name='khans').all()
            
            self.assertEqual(1, len(sets))
            self.assertEqual('KTK', sets[0].code)
            
    def test_all_returns_all_sets(self):
        with vcr.use_cassette('fixtures/all_sets.yaml'):
            sets = Set.all()

            self.assertGreater(len(sets), 190)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>

import json
from urllib.request import Request, urlopen
from urllib.error import HTTPError
from urllib.parse import urlencode


class RestClient(object):
    @staticmethod
    def get(url, params={}):
        """Invoke an HTTP GET request on a url
        
        Args:
            url (string): URL endpoint to request
            params (dict): Dictionary of url parameters 
        Returns:
            dict: JSON response as a dictionary
        """
        request_url = url
        
        if len(params) > 0:
            request_url = "{}?{}".format(url, urlencode(params))

        try:
            req = Request(request_url, headers={'User-Agent': 'Mozilla/5.0'})
            response = json.loads(urlopen(req).read().decode("utf-8"))

            return response
        except HTTPError as err:
            raise MtgException(err.read())


class MtgException(Exception):
    def __init__(self, description):
        self.description = description

    def __str__(self):
        return self.description



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>

import vcr
import unittest
from mtgsdk import Supertype

class TestSupertype(unittest.TestCase):
    def test_all_returns_supertypes(self):
        with vcr.use_cassette('fixtures/supertypes.yaml'):
            supertypes = Supertype.all()
            
            self.assertEqual(["Basic","Legendary","Ongoing","Snow","World"], supertypes)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>

from mtgsdk.config import __version__, __pypi_packagename__, __github_username__, __github_reponame__, __endpoint__
from mtgsdk.card import Card
from mtgsdk.set import Set
from mtgsdk.supertype import Supertype
from mtgsdk.subtype import Subtype
from mtgsdk.type import Type
from mtgsdk.changelog import Changelog
from mtgsdk.restclient import RestClient
from mtgsdk.restclient import MtgException
from mtgsdk.querybuilder import QueryBuilder


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>
from mtgsdk.restclient import RestClient
from mtgsdk.config import __endpoint__


class QueryBuilder(object):
    def __init__(self, type):
        self.params = {}
        self.type = type

    def find(self, id):
        """Get a resource by its id
        
        Args:
            id (string): Resource id    
        Returns:
            object: Instance of the resource type
        """
        url = "{}/{}/{}".format(__endpoint__, self.type.RESOURCE, id)
        response = RestClient.get(url)[self.type.RESOURCE[:-1]]
        return self.type(response)

    def find_many(self, url, type, resource):
        """Get a list of resources
        
        Args:
            url (string): URL to invoke
            type (class): Class type
            resource (string): The REST Resource
        Returns:
            list of object: List of resource instances
        """
        list = []
        response = RestClient.get(url)[resource]
        if len(response) > 0:
            for item in response:
                list.append(type(item))

        return list

    def where(self, **kwargs):
        """Adds a parameter to the dictionary of query parameters
        
        Args:
            **kwargs: Arbitrary keyword arguments.
        Returns:
            QueryBuilder: Instance of the QueryBuilder
        """
        for key, value in kwargs.items():
            self.params[key] = value

        return self

    def all(self):
        """Get all resources, automatically paging through data

        Returns:
            list of object: List of resource objects
        """
        list = [x for x in self.iter()]

        return list
    
    def iter(self):
        """Gets all resources, automating paging through data
        
        Returns:
            iterable of object: Iterable of resource objects
        """
        
        page = 1
        fetch_all = True
        url = "{}/{}".format(__endpoint__, self.type.RESOURCE)

        if 'page' in self.params:
            page = self.params['page']
            fetch_all = False

        while True:
            response = RestClient.get(url, self.params)[self.type.RESOURCE]
            if len(response) > 0:
                for item in response:
                    yield self.type(item)

                if not fetch_all:
                    break
                else:
                    page += 1
                    self.where(page=page)
            else:
                break

        return
    
    def array(self):
        """Get all resources and return the result as an array

        Returns:
            array of str: Array of resources
        """
        url = "{}/{}".format(__endpoint__, self.type.RESOURCE)
        return RestClient.get(url, self.params)[self.type.RESOURCE]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>

import sys, os
from setuptools import setup, find_packages
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'mtgsdk'))
from config import __version__, __pypi_packagename__, __github_username__, __github_reponame__

tests_require = [
    'mock',
    'nose',
    'coverage',
    'yanc',
    'preggy',
    'tox',
    'ipdb',
    'coveralls',
    'sphinx',
    'vcrpy'
]

url='https://github.com/' + __github_username__ + '/' + __github_reponame__
download_url="{}/tarball/{}".format(url, __version__)

setup(
    name=__pypi_packagename__,
    version=__version__,
    description='Magic: The Gathering SDK for magicthegathering.io',
    long_description='''
Magic: The Gathering SDK is a python wrapper around the MTG API located at magicthegathering.io
''',
    keywords='mtg sdk magic gathering gatherer api rest',
    author='Andrew Backes',
    author_email='backes.andrew@gmail.com',
    url=url,
    download_url=download_url,
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3.4',
        'Operating System :: OS Independent',
    ],
    packages=find_packages(),
    include_package_data=False,
    install_requires=[
        # add your dependencies here
        # remember to use 'package-name>=x.y.z,<x.y+1.0' notation (this way you get bugfixes)
    ],
    extras_require={
        'tests': tests_require,
    },
    entry_points={
        'console_scripts': [
            # add cli scripts here in this form:
            # 'mtgsdk=mtgsdk.cli:main',
        ],
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of mtgsdk.
# https://github.com/MagicTheGathering/mtg-sdk-python

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2016, Andrew Backes <backes.andrew@gmail.com>

from mtgsdk.querybuilder import QueryBuilder


class Supertype(object):
    RESOURCE = 'supertypes'

    @staticmethod
    def all():
        return QueryBuilder(Supertype).array()
